LORUM_TEXT = """
Lorem ipsum dolor sit amet
Curbatitur sapien neque

id nisl. Fusce id diam id
-
vitae mi non orci ultricies
""".split(
    "\n"
)

BLOCK_TEXT = r"""
 <- Cursor, move with 'j' 🧱.
                          🧱.
                          🧱.
                          🧱.
                          🧱.
                          🧱.
🧱🧱🧱🧱🧱🧱             🧱.
Blocked ?. Try 'l'          .
                            .
                            .
                            .
                            .
                            .
                            .
                            .
                            .
              🧱🧱🧱🧱🧱🧱 .
      Again ? Maybe with 'h'.
                            .
                            .
                            .
                            .
    🧱🧱🧱🧱🧱🧱🧱🧱🧱🧱🧱.
    🧱                      .
    🧱   ^     🧱           .
    🧱   |     🧱           .
    🧱   k     🧱           .
               🧱  goal->  🍩.
""".split(
    "\n"
)[
    1:-1
]
